package com.example.wanandroidformine.main

import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.KeyEvent
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentTransaction
import com.blankj.utilcode.util.ToastUtils
import com.example.wanandroidformine.AppManager
import com.example.wanandroidformine.BlankFragment
import com.example.wanandroidformine.R
import com.example.wanandroidformine.base.BaseActivity
import com.example.wanandroidformine.main.home.HomeFragment
import com.example.wanandroidformine.main.mine.MineFragment
import com.example.wanandroidformine.main.tab.TabEntity
import com.flyco.tablayout.listener.CustomTabEntity
import com.flyco.tablayout.listener.OnTabSelectListener
import kotlinx.android.synthetic.main.activity_main.*
import java.util.*
import kotlin.system.exitProcess

class MainActivity : BaseActivity() {
    //Fragment 栈管理
    private val mStack = Stack<Fragment>()
    private val mHomeFragment :HomeFragment=HomeFragment()
    private val mDiscoveryFragment by lazy { BlankFragment.getInstance(mTitles[1]) }
    private val mHotFragment by lazy { BlankFragment.getInstance(mTitles[2]) }

    private val  mMineFragment:MineFragment=MineFragment()
    private val mTitles = arrayOf("每日精选", "发现", "热门", "我的")
    private var mExitTime: Long = 0
    private val mTabEntities = ArrayList<CustomTabEntity>()
    // 未被选中的图标
    private val mIconUnSelectIds = intArrayOf(R.mipmap.ic_home_normal, R.mipmap.ic_discovery_normal, R.mipmap.ic_hot_normal, R.mipmap.ic_mine_normal)
    // 被选中的图标
    private val mIconSelectIds = intArrayOf(R.mipmap.ic_home_selected, R.mipmap.ic_discovery_selected, R.mipmap.ic_hot_selected, R.mipmap.ic_mine_selected)
    //默认为0
    private var mIndex = 0


    override fun onCreate(savedInstanceState: Bundle?) {
        savedInstanceState?.let {
            mIndex = it.getInt("currTabIndex")
        }

        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        initFragment()
        initTab()
        tab_layout.currentTab = mIndex
        switchFragment(mIndex)
        initMsgAndDot()
    }



    /**
     * 初始化Fragment栈管理
     */
    private fun initFragment() {
        val manager = supportFragmentManager.beginTransaction()
        manager.add(R.id.fl_container,mHomeFragment)
        manager.add(R.id.fl_container,mDiscoveryFragment)
        manager.add(R.id.fl_container,mHotFragment)
        manager.add(R.id.fl_container,mMineFragment)
        manager.commit()
        mStack.add(mHomeFragment)
        mStack.add(mDiscoveryFragment)
        mStack.add(mHotFragment)
        mStack.add(mMineFragment)
    }

    //初始化底部菜单
    private fun initTab() {
        (0 until mTitles.size)
            .mapTo(mTabEntities) { TabEntity(mTitles[it], mIconSelectIds[it], mIconUnSelectIds[it]) }
        //为Tab赋值
        tab_layout.setTabData(mTabEntities)
        tab_layout.setOnTabSelectListener(object : OnTabSelectListener {
            override fun onTabSelect(position: Int) {
                //切换Fragment
                switchFragment(position)

            }

            override fun onTabReselect(position: Int) {
            }

        })
    }
    /**
     *切换Fragment
     * @param position 下标
     */
    private fun switchFragment(position: Int) {
        val manager = supportFragmentManager.beginTransaction()
        hideFragments(manager)
        manager.show(mStack[position])
        mIndex = position
        tab_layout.currentTab = mIndex
        manager.commit()
    }

    /**
     * 隐藏所有的Fragment
     */
    private fun hideFragments(manager: FragmentTransaction) {
        for (fragment in mStack){
            manager.hide(fragment)
        }
    }
    /**
     * 给tab增加未读消息或者红点提示
     */
    private fun initMsgAndDot() {
        tab_layout.showMsg(3, 1)
        tab_layout.showDot(1)
    }

    /**
     * 退出
     */
    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        if (keyCode == KeyEvent.KEYCODE_BACK)
            forClick() //调用双击退出函数
        return false
    }
    var isExit: Boolean = false
    private fun forClick() {
        val tExit: Timer
        if (!isExit) {
            isExit = true // 准备退出
            ToastUtils.showShort("再按一次退出程序")
            tExit = Timer()
            tExit.schedule(object : TimerTask() {
                override fun run() {
                    isExit = false
                }
            }, 1500) // 1.5秒 时间可以自己设定
        } else {
           val appM :AppManager= AppManager()
            appM.finishAllActivity()
            finish()
            exitProcess(0)//finish();
        }
    }


}