package com.example.wanandroidformine.main.home

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout
import com.blankj.utilcode.util.ToastUtils
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.RoundedCorners
import com.bumptech.glide.request.RequestOptions
import com.example.wanandroidformine.R
import com.example.wanandroidformine.adapter.HomeListAdapter
import com.example.wanandroidformine.base.BaseFragment
import com.example.wanandroidformine.javaBean.BannerBean
import com.example.wanandroidformine.javaBean.HomeData
import com.example.wanandroidformine.javaBean.HomeListBean
import com.example.wanandroidformine.net.schedules.SchedulerProvider
import com.orhanobut.logger.Logger
import com.youth.banner.Banner
import com.youth.banner.adapter.BannerImageAdapter
import com.youth.banner.holder.BannerImageHolder
import com.youth.banner.indicator.CircleIndicator
import kotlinx.android.synthetic.main.fragment_home.*
import kotlinx.android.synthetic.main.toolbar_layout_noback.*


class HomeFragment : BaseFragment() ,HomeContract.View{
    private val presenter: HomePresenter by lazy {
        HomePresenter(SchedulerProvider.getInstatnce()!!, this)
    }

    private var homeListAdapter: HomeListAdapter? = null

    private var  homeDatas: MutableList<HomeData> = mutableListOf()

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        return inflater.inflate(R.layout.fragment_home, container, false)
    }

    override fun initTitle() {
        mTitleBar.setTitle(R.string.home)
    }

    override fun initView() {




        var banner: Banner<DataBean, BannerImageAdapter<DataBean>> = banner as Banner<DataBean, BannerImageAdapter<DataBean>>

        banner.setAdapter(object : BannerImageAdapter<DataBean>(DataBean.testData3) {
            override fun onBindView(holder: BannerImageHolder, data: DataBean, position: Int, size: Int) {
                //图片加载自己实现
                Glide.with(holder.itemView)
                    .load(data.imageUrl)
                    .apply(RequestOptions.bitmapTransform(RoundedCorners(1)))
                    .into(holder.imageView) }
        }).addBannerLifecycleObserver(this).setIndicator(CircleIndicator(context))

        banner.setOnBannerListener { data, position ->
            data as DataBean
            ToastUtils.showShort("点击了---"+data.imageUrl)

        }
//        presenter.getBannerData(true)
        presenter.getHomeList(true)
        homeListAdapter = HomeListAdapter(R.layout.homelist_item,homeDatas)




        recyclerView.layoutManager = LinearLayoutManager(context, LinearLayoutManager.VERTICAL, false)
        recyclerView.adapter = homeListAdapter


        swipeRefreshLayout.setOnRefreshListener(object : SwipeRefreshLayout.OnRefreshListener{
            override fun onRefresh() {
                presenter.getHomeList(true)
            }


        })

        recyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
                super.onScrollStateChanged(recyclerView, newState)
                if (newState == RecyclerView.SCROLL_STATE_IDLE) {
                    val lastVisiblePosition: Int = (recyclerView.layoutManager as LinearLayoutManager).findLastVisibleItemPosition()
                    if (lastVisiblePosition >= (recyclerView.layoutManager as LinearLayoutManager).getItemCount() - 1) {

                        presenter.getHomeList(false)

                    }
                }
            }
        })

    }

    override fun getHomeListSuccess(homeListBean: HomeListBean, isRefresh: Boolean) {

        if (isRefresh){
            homeDatas.clear()
            homeDatas.addAll(homeListBean.datas as MutableList<HomeData>)
            swipeRefreshLayout?.let {
                it.setRefreshing(false)
            }
        }else{

            homeDatas.addAll(homeListBean.datas as MutableList<HomeData>)
        }
        homeListAdapter?.notifyDataSetChanged()
    }

    override fun getHomeListFail(msg: String, isRefresh: Boolean) {
        Logger.e("getHomeListFail----------"+msg)

    }

    override fun showBanner(banners: List<BannerBean>, isRefresh: Boolean) {
     Logger.e(banners.toString())
    }

    override fun getBannerFail(errorMsg: String, isRefresh: Boolean) {
        Logger.e("getBannerFail------------"+errorMsg)
    }


}